@@ -263,7 +263,10 @@ static int verify_directory_item (reiserfs_filsys_t * fs, struct buffer_head * b
  
     if ( (get_ih_entry_count (ih) > (get_ih_item_len(ih) / (DEH_SIZE + min_entry_size))) ||
           (get_ih_entry_count (ih) == 0))
+    {
         set_ih_entry_count (ih, (int)get_ih_item_len(ih) / (DEH_SIZE + min_entry_size));
+        mark_buffer_dirty (bh);
+    }
 
     if (get_ih_entry_count (ih) == 0) {
 	delete_item (fs, bh, item_num);
@@ -362,8 +365,7 @@ static int verify_directory_item (reiserfs_filsys_t * fs, struct buffer_head * b
 	    hash_code = 0;
 
 	    for (j = min_length; j <= max_length; j ++) {
-		hash_code = find_hash_in_use (name, j,
-					      GET_HASH_VALUE (get_deh_offset (deh + i)),
+		hash_code = find_hash_in_use (name, j, get_deh_offset (deh + i),
 					      get_sb_hash_code (fs->fs_ondisk_sb));
 /*		add_hash_hit (fs, hash_code);*/
 		if (code2func (hash_code) != 0) {
@@ -524,9 +526,11 @@ static int verify_directory_item (reiserfs_filsys_t * fs, struct buffer_head * b
 	}
     }
     
-    if (bad == get_ih_entry_count (&tmp) && lost_found != bad) {	
-	fsck_log ("%s: block %lu, item %H: All entries were deleted from the directory\n", 
-	    __FUNCTION__, bh->b_blocknr, &tmp);	
+    if (bad == get_ih_entry_count (&tmp)) {
+	if (lost_found != bad) {
+	    fsck_log ("%s: block %lu, item %H: All entries were deleted from the directory\n", 
+		      __FUNCTION__, bh->b_blocknr, &tmp);	
+	}
 	return -1;
     }
 
@@ -561,7 +565,8 @@ static int is_wrong_short_key (struct key * key) {
     if (get_key_dirid (key) == 0 || get_key_objectid (key) == 0 || get_key_objectid (key) == 1 ||
 	get_key_dirid (key) == ~(__u32)0 || get_key_objectid (key) == ~(__u32)0 ||
 	get_key_dirid (key) == get_key_objectid (key) ||
-	(get_key_dirid (key) == 1 && get_key_objectid (key) != 2) ||
+	/* the alloc=packing_groups used to allow dirid = 1
+	(get_key_dirid (key) == 1 && get_key_objectid (key) != 2) || */
 	(get_key_dirid (key) != 1 && get_key_objectid (key) == 2) )
 	return 1;
 
@@ -582,6 +587,9 @@ int leaf_structure_check (reiserfs_filsys_t * fs, struct buffer_head * bh) {
 	return 1;
     }
 
+    if (get_blkh_nr_items (blkh) == 0)
+	return 0;
+    
     counted = leaf_count_ih(bh->b_data, bh->b_size);
 
     if (counted < get_blkh_nr_items (blkh)) {
@@ -1285,7 +1293,7 @@ static void pass0_correct_leaf (reiserfs_filsys_t * fs,
 */
 	ind_item = (__u32 *)B_I_PITEM (bh, ih);
 	for (j = 0; j < (int)I_UNFM_NUM (ih); j ++) {
-	    unfm_ptr = le32_to_cpu (ind_item [j]);
+	    unfm_ptr = d32_get (ind_item, j);
 	    if (!unfm_ptr)
 		continue;
 #if 0
@@ -1309,7 +1317,7 @@ static void pass0_correct_leaf (reiserfs_filsys_t * fs,
 		fsck_log ("pass0: %d-th pointer (%lu) in item %k (leaf block %lu) is wrong\n",
 			  j, unfm_ptr, &ih->ih_key, bh->b_blocknr);
 		*/
-		ind_item [j] = 0;
+		d32_put(ind_item, j, 0);
 		dirty = 1;
 		continue;
 	    }
@@ -1436,10 +1444,10 @@ static int is_bad_indirect (struct item_head * ih, char * item, int dev, int blo
     for (i = 0; i < I_UNFM_NUM (ih); i ++) {
 	__u32 * ind = (__u32 *)item;
 
-	if (le32_to_cpu (ind[i]) >= blocks) {
+	if (d32_get (ind, i) >= blocks) {
 	    bad ++;
 	    fsck_log ("is_bad_indirect: %d-th pointer of item %H looks bad (%lu)\n",
-		      i, ih, le32_to_cpu (ind [i]));
+		      i, ih, d32_get (ind, i));
 	    continue;
 	}
     }
@@ -1766,12 +1774,19 @@ static void init_source_bitmap (reiserfs_filsys_t * fs)
 
     case EXTERN_BITMAP:
 	fp = fopen (fsck_data (fs)->rebuild.bitmap_file_name, "r");
-	if (!fp)
-	    reiserfs_panic ("Could not load bitmap: %s\n", strerror(errno));
+	
+	if (!fp) {
+	    reiserfs_exit (EXIT_OPER, "Could not load bitmap: %s\n", 
+			   strerror(errno));
+	}
+	
 	fsck_source_bitmap (fs) = reiserfs_bitmap_load (fp);
-	if (!fsck_source_bitmap (fs))
-	    reiserfs_panic ("Could not load fitmap from \"%s\"", 
-			    fsck_data (fs)->rebuild.bitmap_file_name);
+	
+	if (!fsck_source_bitmap (fs)) {
+	    reiserfs_exit (EXIT_OPER, "Could not load fitmap from \"%s\"", 
+			   fsck_data (fs)->rebuild.bitmap_file_name);
+	}
+
 	fsck_progress ("%d blocks marked used in extern bitmap\n", 
 		       reiserfs_bitmap_ones (fsck_source_bitmap (fs)));
 	fclose (fp);
@@ -1963,7 +1978,10 @@ void pass_0 (reiserfs_filsys_t * fs)
 {
     if (get_reiserfs_format (fs->fs_ondisk_sb) != fs->fs_format || 
         get_reiserfs_format (fs->fs_ondisk_sb) == REISERFS_FORMAT_UNKNOWN)
-	reiserfs_panic ("pass 0: ReiserFS format version mismatch found, you should run --rebuild-sb");
+    {
+	reiserfs_exit (EXIT_OPER, "pass 0: ReiserFS format version mismatch "
+		       "found, you should run --rebuild-sb");
+    }
   
     fsck_progress ("\nPass 0:\n");
     if (fsck_log_file (fs) != stderr)
